@@ -26,3 +26,7 @@
 #undef  ALIGNOF_CORBA_UNSIGNED_SHORT
 #undef  ALIGNOF_CORBA_WCHAR
 #undef  ALIGNOF_CORBA_POINTER
+
+/* TCP wrappers */
+#undef HAVE_HOSTS_ACCESS
+#undef HAVE_TCPD_H
