@@ -108,6 +108,12 @@ typedef struct _GIOPMessage
   } u;
 } GIOPMessage;
 
+typedef enum {
+  GIOP_MSG_READING_HEADER,
+  GIOP_MSG_READING_BODY,
+  GIOP_MSG_READY
+} GIOPMessageBufferState;
+
 #define GIOP_RECV_BUFFER(x) ((GIOPRecvBuffer *)x)
 typedef struct _GIOPRecvBuffer
 {
@@ -118,6 +124,9 @@ typedef struct _GIOPRecvBuffer
   gpointer cur;
 
   void (*decoder)(gpointer dest, gpointer src, gulong len);
+
+  GIOPMessageBufferState state;
+  gint left_to_read;
 } GIOPRecvBuffer;
 
 /* This function needs to be called before useful things happen */
@@ -211,6 +220,6 @@ extern char giop_scratch_space[2048];
 gulong giop_message_buffer_do_alignment(GIOPMessageBuffer *buffer,
 					gulong align_for);
 
-gboolean giop_msg_conversion_needed(GIOPMessageBuffer *msgbuf);
+#define giop_msg_conversion_needed(msgbuf) (conversion_needed(GIOP_MESSAGE_BUFFER(msgbuf)->message_header.flags & 1))
 
 #endif /* GIOP_MSG_BUFFER_H */
