@@ -68,7 +68,11 @@ static void cbe_hdr_do_native(CBEHeaderInfo *hinfo);
 void
 orbit_cbe_write_header(FILE *outfile, IDL_ns ns, IDL_tree tree)
 {
-  CBEHeaderInfo hinfo = {outfile, ns, tree};
+  CBEHeaderInfo hinfo;
+
+  hinfo.of = outfile;
+  hinfo.ns = ns;
+  hinfo.tree = tree;
 
   g_return_if_fail(IDL_NODE_TYPE(tree) == IDLN_LIST);
 
@@ -401,7 +405,7 @@ cbe_hdr_do_type_dcl(CBEHeaderInfo *hinfo)
       fprintf(hinfo->of, " %s;\n", id);
       fprintf(hinfo->of, "#endif /* !_%s_defined */\n\n", id);
       fprintf(hinfo->of, "extern const struct CORBA_TypeCode_struct TC_%s_struct;\n", id);
-      fprintf(hinfo->of, "extern const CORBA_TypeCode TC_%s;\n", id);
+      fprintf(hinfo->of, "#define TC_%s (&TC_%s_struct)\n", id, id);
 
       switch(IDL_NODE_TYPE(IDL_TYPE_DCL(hinfo->tree).type_spec)) {
 
@@ -466,18 +470,23 @@ cbe_hdr_do_except_dcl(CBEHeaderInfo *hinfo)
 		     "#define _%s_defined\n\n", id, id);
   fprintf(hinfo->of, "typedef struct %s {\n", id);
   
-  for(curitem = IDL_EXCEPT_DCL(hinfo->tree).members; curitem;
-      curitem = IDL_LIST(curitem).next) {
-    child_hinfo.tree = IDL_LIST(curitem).data;
-    orbit_cbe_hdr_process_piece(&child_hinfo);
-    fprintf(hinfo->of, "\n");
+  if (IDL_EXCEPT_DCL(hinfo->tree).members) {
+    for(curitem = IDL_EXCEPT_DCL(hinfo->tree).members; curitem;
+        curitem = IDL_LIST(curitem).next) {
+      child_hinfo.tree = IDL_LIST(curitem).data;
+      orbit_cbe_hdr_process_piece(&child_hinfo);
+      fprintf(hinfo->of, "\n");
+    }
   }
+  else
+    fprintf(hinfo->of, "	int empty_struct_member;\n");
+
   fprintf(hinfo->of, "} %s;\n", id);
   fprintf(hinfo->of, "#define ex_%s \"%s\"\n", id,
 	  IDL_IDENT(IDL_EXCEPT_DCL(hinfo->tree).ident).repo_id);
   fprintf(hinfo->of, "#endif /* !_%s_defined */\n\n", id);
   fprintf(hinfo->of, "extern const struct CORBA_TypeCode_struct TC_%s_struct;\n", id);
-  fprintf(hinfo->of, "extern const CORBA_TypeCode TC_%s;\n", id);
+  fprintf(hinfo->of, "#define TC_%s (&TC_%s_struct)\n", id, id);
   fprintf(hinfo->of, "extern %s* %s__alloc(void);\n", id, id);
   fprintf(hinfo->of, "extern gpointer %s__free(gpointer mem, gpointer dat, CORBA_boolean free_strings); /* ORBit internals */\n", id);
 
@@ -655,6 +664,9 @@ cbe_hdr_do_param_dcl(CBEHeaderInfo *hinfo)
 {
   IDL_ParamRole r = DATA_IN;
 
+  if(IDL_PARAM_DCL(hinfo->tree).attr == IDL_PARAM_IN)
+    fprintf(hinfo->of, "const ");
+
   orbit_cbe_write_typespec(hinfo->of,
 			   IDL_PARAM_DCL(hinfo->tree).param_type_spec);
 
@@ -820,7 +832,7 @@ cbe_hdr_do_type_enum(CBEHeaderInfo *hinfo)
   fprintf(hinfo->of, "} %s;\n", enumid);
   fprintf(hinfo->of, "#endif /* !_%s_defined */\n\n", enumid);
   fprintf(hinfo->of, "extern const struct CORBA_TypeCode_struct TC_%s_struct;\n", enumid);
-  fprintf(hinfo->of, "extern const CORBA_TypeCode TC_%s;\n", enumid);
+  fprintf(hinfo->of, "#define TC_%s (&TC_%s_struct)\n", enumid, enumid);
 
   free(enumid);
 }
@@ -859,7 +871,7 @@ cbe_hdr_do_type_array(CBEHeaderInfo *hinfo)
   /* The ifdef corresponding to this was written in cbe_hdr_do_type_dcl */
   fprintf(hinfo->of, "#endif /* !_%s_defined */\n\n", id);
   fprintf(hinfo->of, "extern const struct CORBA_TypeCode_struct TC_%s_struct;\n", id);
-  fprintf(hinfo->of, "extern const CORBA_TypeCode TC_%s;\n", id);
+  fprintf(hinfo->of, "#define TC_%s (&TC_%s_struct)\n", id, id);
   fprintf(hinfo->of, "extern %s_slice* %s__alloc(void);\n", id, id);
   fprintf(hinfo->of, "extern gpointer %s__free(gpointer mem, gpointer dat, CORBA_boolean free_strings); /* ORBit internals */\n", id);
 
@@ -878,17 +890,21 @@ cbe_hdr_do_type_struct(CBEHeaderInfo *hinfo)
 		     "#define _%s_defined\n\n", id, id);
   fprintf(hinfo->of, "typedef struct %s {\n", id);
 
-  for(curitem = IDL_TYPE_STRUCT(hinfo->tree).member_list; curitem;
-      curitem = IDL_LIST(curitem).next) {
-    child_hinfo.tree = IDL_LIST(curitem).data;
-    orbit_cbe_hdr_process_piece(&child_hinfo);
-    fprintf(hinfo->of, "\n");
-  }
+  if (IDL_TYPE_STRUCT(hinfo->tree).member_list) {
+    for(curitem = IDL_TYPE_STRUCT(hinfo->tree).member_list; curitem;
+        curitem = IDL_LIST(curitem).next) {
+      child_hinfo.tree = IDL_LIST(curitem).data;
+      orbit_cbe_hdr_process_piece(&child_hinfo);
+      fprintf(hinfo->of, "\n");
+    }
+  } 
+  else
+    fprintf(hinfo->of, "	int empty_struct_member;\n");
 
   fprintf(hinfo->of, "} %s;\n", id);
   fprintf(hinfo->of, "#endif /* !_%s_defined */\n\n", id);
   fprintf(hinfo->of, "extern const struct CORBA_TypeCode_struct TC_%s_struct;\n", id);
-  fprintf(hinfo->of, "extern const CORBA_TypeCode TC_%s;\n", id);
+  fprintf(hinfo->of, "#define TC_%s (&TC_%s_struct)\n", id, id);
   fprintf(hinfo->of, "extern %s* %s__alloc(void);\n", id, id);
   fprintf(hinfo->of, "extern gpointer %s__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);\n", id);
   free(id);
@@ -927,7 +943,7 @@ cbe_hdr_do_type_union(CBEHeaderInfo *hinfo)
   fprintf(hinfo->of, "  } _u;\n} %s;\n", id);
   fprintf(hinfo->of, "#endif /* !_%s_defined */\n\n", id);
   fprintf(hinfo->of, "extern const struct CORBA_TypeCode_struct TC_%s_struct;\n", id);
-  fprintf(hinfo->of, "extern const CORBA_TypeCode TC_%s;\n", id);
+  fprintf(hinfo->of, "#define TC_%s (&TC_%s_struct)\n", id, id);
   fprintf(hinfo->of, "extern %s* %s__alloc(void);\n", id, id);
   fprintf(hinfo->of, "extern gpointer %s__free(gpointer mem, gpointer dat, CORBA_boolean free_strings); /* ORBit internals */\n", id);
 
@@ -1037,7 +1053,7 @@ cbe_hdr_interface_print_epv_for_attr(CBEHeaderInfo *hinfo)
 	    IDL_IDENT(IDL_LIST(curitem).data).str);
 
     if(!IDL_ATTR_DCL(hinfo->tree).f_readonly) {
-      fprintf(hinfo->of, "  void (*_set_%s)(PortableServer_Servant servant, ",
+      fprintf(hinfo->of, "  void (*_set_%s)(PortableServer_Servant servant, const ",
 	      IDL_IDENT(IDL_LIST(curitem).data).str);
       
       orbit_cbe_write_typespec(hinfo->of,
@@ -1071,6 +1087,7 @@ cbe_hdr_interface_print_epv(CBEHeaderInfo *hinfo)
       cbe_hdr_interface_print_epv_for_attr(&subhinfo);
       break;
     default:
+      break;
     }
   }
 }
@@ -1119,6 +1136,7 @@ cbe_hdr_interface_print_inheritance_ops(IDL_tree node, CBEHeaderInfo *current)
       cbe_hdr_do_inherited_attr_dcl(&subhinfo, current->tree);
       break;
     default:
+      break;
     }
   }
 }
@@ -1142,8 +1160,7 @@ cbe_hdr_do_interface(CBEHeaderInfo *hinfo)
   fprintf(hinfo->of, "extern CORBA_unsigned_long %s__classid;\n", id);
   fprintf(hinfo->of, "extern const struct CORBA_TypeCode_struct TC_%s_struct;\n",
 	  id);
-  fprintf(hinfo->of, "extern const CORBA_TypeCode TC_%s;\n",
-	  id);
+  fprintf(hinfo->of, "#define TC_%s (&TC_%s_struct)\n", id, id);
   subhinfo.tree = IDL_INTERFACE(hinfo->tree).body;
   cbe_hdr_do_list(&subhinfo);
 
