@@ -11,6 +11,18 @@ typedef struct {
   char *free_internal;
 } CBESkelInfo;
 
+typedef struct {
+  char *iface_id;
+  char *opname;
+  IDL_tree op;
+} CBESkelOpInfo;
+
+typedef struct {
+  CBESkelInfo *ski;
+  GSList *oplist;
+  gint curlevel;
+} CBESkelInterfaceTraverseInfo;
+
 static void orbit_cbe_skel_process_piece(CBESkelInfo *ski);
 static void cbe_skel_do_attr_dcl(CBESkelInfo *ski);
 static void cbe_skel_do_inherited_attr_dcl(CBESkelInfo *ski,
@@ -65,14 +77,19 @@ void
 orbit_cbe_write_skeletons(FILE *outfile, IDL_ns ns, IDL_tree tree,
 			  const char *header_filename)
 {
-  CBESkelInfo ski = {outfile, ns, tree, "CORBA_TRUE"};
+  CBESkelInfo ski;
+
+  ski.of = outfile;
+  ski.ns = ns;
+  ski.tree = tree;
+  ski.free_internal = "CORBA_TRUE";
 
   fprintf(outfile, "/*\n"
   		   " * This file was generated by orbit-idl - DO NOT EDIT!\n"
 		   " */\n\n");
   fprintf(outfile, "#include <string.h>\n");
   fprintf(outfile, "#include \"%s\"\n", header_filename);
-  fprintf(outfile, "#define GET_ATOM(x) ({ GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->decoder(&x, (GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur), sizeof(x)); GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur += sizeof(x); })\n\n");
+  fprintf(outfile, "#define GET_ATOM(x) G_STMT_START{ GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->decoder(&x, (GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur), sizeof(x)); ((guchar *)GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur) += sizeof(x); }G_STMT_END\n\n");
 
   orbit_cbe_skel_process_piece(&ski);
 }
@@ -295,40 +312,42 @@ cbe_skel_do_attr_dcl(CBESkelInfo *ski)
   cbe_skel_do_attr_dcl_internal(ski, NULL, FALSE);
 }
 
+#if 0
 static void
 cbe_skel_do_inherited_attr_dcl(CBESkelInfo *ski, IDL_tree current_interface)
 {
   cbe_skel_do_attr_dcl_internal(ski, current_interface, TRUE);
 }
+#endif
 
 static void
 cbe_skel_do_binop(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_boolean(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_case_stmt(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_char(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_const_dcl(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
@@ -374,31 +393,31 @@ cbe_skel_do_fixed(CBESkelInfo *ski)
 static void
 cbe_skel_do_float(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_forward_dcl(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_gentree(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_ident(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_integer(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 void
@@ -426,10 +445,76 @@ cbe_skel_print_skelptr(FILE *of, IDL_tree tree)
     }
     break;
   default:
+    break;
   }
   free(id);
 }
 
+static gint
+cbe_skel_compare_op_dcls(CBESkelOpInfo *op1, CBESkelOpInfo *op2)
+{
+  return strcmp(op1->opname, op2->opname);
+}
+
+static void
+cbe_skel_free_op_info(CBESkelOpInfo *op)
+{
+  g_free(op->opname);
+  g_free(op->iface_id);
+  g_free(op);
+}
+
+static void
+cbe_skel_interface_add_relayer(IDL_tree intf, CBESkelInterfaceTraverseInfo *iti)
+{
+  CBESkelOpInfo *newopi;
+  IDL_tree curitem, curdcl, curattr, curattrdcl;
+  char *iface_id;
+
+  iface_id =
+    IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(IDL_INTERFACE(intf).ident),
+			    "_", 0);
+
+  for(curitem = IDL_INTERFACE(intf).body; curitem;
+      curitem = IDL_LIST(curitem).next) {
+    curdcl = IDL_LIST(curitem).data;
+
+    switch(IDL_NODE_TYPE(curdcl)) {
+    case IDLN_OP_DCL:
+      newopi = g_new0(CBESkelOpInfo, 1);
+      newopi->iface_id = g_strdup(iface_id);
+      newopi->opname = g_strdup(IDL_IDENT(IDL_OP_DCL(curdcl).ident).str);
+      iti->oplist = g_slist_insert_sorted(iti->oplist, newopi,
+					  (GCompareFunc)cbe_skel_compare_op_dcls);
+      break;
+    case IDLN_ATTR_DCL:
+      for(curattr = IDL_ATTR_DCL(curdcl).simple_declarations;
+	  curattr; curattr = IDL_LIST(curattr).next) {
+	curattrdcl = IDL_LIST(curattr).data;
+
+	newopi = g_new0(CBESkelOpInfo, 1);
+	newopi->iface_id = g_strdup(iface_id);
+	newopi->opname = g_strdup_printf("_get_%s", IDL_IDENT(curattrdcl).str);
+	iti->oplist = g_slist_insert_sorted(iti->oplist, newopi,
+					    (GCompareFunc)cbe_skel_compare_op_dcls);
+	if(!IDL_ATTR_DCL(curdcl).f_readonly) {
+	  newopi = g_new0(CBESkelOpInfo, 1);
+	  newopi->iface_id = g_strdup(iface_id);
+	  newopi->opname = g_strdup_printf("_set_%s", IDL_IDENT(curattrdcl).str);
+	  iti->oplist = g_slist_insert_sorted(iti->oplist, newopi,
+					      (GCompareFunc)cbe_skel_compare_op_dcls);
+	}
+      }
+      break;
+    default:
+      break;
+    }
+  }
+
+  free(iface_id);
+}
+
+#if 0
 static void
 cbe_skel_interface_print_relayer_internal(CBESkelInfo *ski, IDL_tree curitem, char *iface_id, char *cur_id)
 {
@@ -475,6 +560,7 @@ cbe_skel_interface_print_relayer_internal(CBESkelInfo *ski, IDL_tree curitem, ch
     }
     break;
   default:
+    break;
   }
 }
 
@@ -507,34 +593,76 @@ cbe_skel_interface_print_inheritance_relayer(IDL_tree node, CBESkelInfo *current
   free(iface_id);
   free(cur_id);
 }
+#endif
 
-void
+static void
+cbe_skel_interface_print_relayers(const CBESkelInterfaceTraverseInfo *iti)
+{
+  CBESkelInterfaceTraverseInfo subiti = *iti;
+  GSList *curnode;
+  CBESkelOpInfo *opi;
+  char curchar;
+
+  curnode = iti->oplist;
+  subiti.curlevel = iti->curlevel+1;
+  fprintf(iti->ski->of, "switch(opname[%d]) {\n", iti->curlevel);
+  while(curnode) {
+    opi = (CBESkelOpInfo *)curnode->data;
+    if(iti->curlevel > strlen(opi->opname)) {
+      curnode = g_slist_next(curnode);
+      continue;
+    }
+    curchar = opi->opname[iti->curlevel];
+    if(curchar)
+      fprintf(iti->ski->of, "case '%c':\n", curchar);
+    else
+      fprintf(iti->ski->of, "case '\\0':\n");
+    subiti.oplist = NULL;
+    while(curnode && ((CBESkelOpInfo *)curnode->data)->opname[iti->curlevel]
+	  == curchar) {
+      subiti.oplist = g_slist_append(subiti.oplist, curnode->data);
+      curnode = g_slist_next(curnode);
+    }
+
+    if(g_slist_length(subiti.oplist) > 1) {
+      if(curchar)
+	cbe_skel_interface_print_relayers(&subiti);
+      else
+	g_error("two ops with same name!!!!");
+    } else {
+       fprintf(iti->ski->of, "*impl = servant->vepv->%s_epv->%s;\n",
+	       opi->iface_id, opi->opname);
+       fprintf(iti->ski->of, "return (gpointer)skel_%s_%s;\n",
+	       opi->iface_id, opi->opname);
+    }
+    fprintf(iti->ski->of, "break;\n");
+    g_slist_free(subiti.oplist);
+  }
+  fprintf(iti->ski->of, "default: return NULL;\n}\n");
+}
+
+static void
 cbe_skel_interface_print_relayer(CBESkelInfo *ski)
 {
-  /* XXX Prime candidate for optimization here. Is using gperf reasonable?
-     In any case, there IS a better way ;-)
-     -- Elliot */
   char *id;
-  IDL_tree curitem;
+  CBESkelInterfaceTraverseInfo iti;
 
   id = IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(IDL_INTERFACE(ski->tree).ident), "_", 0);
-  fprintf(ski->of, "static ORBitSkeleton get_skel_%s(POA_%s *servant,\nGIOPRecvBuffer *_ORBIT_recv_buffer,\ngpointer *epv)\n", id, id);
+  fprintf(ski->of, "static ORBitSkeleton get_skel_%s(POA_%s *servant,\nGIOPRecvBuffer *_ORBIT_recv_buffer,\ngpointer *impl)\n", id, id);
   fprintf(ski->of, "{\n");
+  fprintf(ski->of, "gchar *opname = _ORBIT_recv_buffer->message.u.request.operation;\n\n");
 
-  for(curitem = IDL_INTERFACE(ski->tree).body;
-      curitem; curitem = IDL_LIST(curitem).next) {
-    cbe_skel_interface_print_relayer_internal(ski, curitem, id, id);
-  }
+  iti.ski = ski;
+  iti.oplist = NULL;
+  iti.curlevel = 0;
 
-  free(id);
+  IDL_tree_traverse_parents(ski->tree,
+			    (GFunc)cbe_skel_interface_add_relayer, &iti);
 
-  IDL_tree_traverse_parents(ski->tree, (gpointer)cbe_skel_interface_print_inheritance_relayer, ski);
-
-  fprintf(ski->of, "  {\n");
-  fprintf(ski->of, "    g_error(\"Unknown operation\");\n");
-  fprintf(ski->of, "  }\n");
-  fprintf(ski->of, "  return NULL;\n");
+  cbe_skel_interface_print_relayers(&iti);
 
+  g_slist_foreach(iti.oplist, (GFunc)cbe_skel_free_op_info, NULL);
+  g_slist_free(iti.oplist);
 
   fprintf(ski->of, "}\n\n");
 }
@@ -576,6 +704,7 @@ cbe_skel_interface_print_objref_initializer(CBESkelInfo *ski)
   fprintf(ski->of, "}\n");
 }
 
+#if 0
 static void
 cbe_skel_interface_print_inheritance_ops(IDL_tree node, CBESkelInfo *current)
 {
@@ -611,9 +740,11 @@ cbe_skel_interface_print_inheritance_ops(IDL_tree node, CBESkelInfo *current)
       cbe_skel_do_inherited_attr_dcl(&subski, current->tree);
       break;
     default:
+      break;
     }
   }
 }
+#endif
 
 static void
 cbe_skel_do_interface(CBESkelInfo *ski)
@@ -629,12 +760,15 @@ cbe_skel_do_interface(CBESkelInfo *ski)
 
   id = IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(IDL_INTERFACE(ski->tree).ident), "_", 0);
 
+#if 0
+  /* useless... */
   subski.tree = IDL_INTERFACE(ski->tree).inheritance_spec;
   if(subski.tree != NULL) {
     fprintf(ski->of, "/* Begin inherited operations for %s */\n\n", id);
     IDL_tree_traverse_parents(subski.tree, (gpointer)cbe_skel_interface_print_inheritance_ops, ski);
     fprintf(ski->of, "/* End inherited operations for %s */\n\n", id);
   }
+#endif
 
   cbe_skel_interface_print_relayer(ski);
 
@@ -869,7 +1003,7 @@ cbe_skel_do_op_dcl(CBESkelInfo *ski)
   g_assert(curitem);
   id2 = IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(IDL_INTERFACE(curitem).ident), "_", 0);
 
-  fprintf(ski->of, "\nvoid\nskel_%s(POA_%s *_ORBIT_servant,\nGIOPRecvBuffer *_ORBIT_recv_buffer,\nCORBA_Environment *ev,\n", id, id2);
+  fprintf(ski->of, "\nstatic void\nskel_%s(POA_%s *_ORBIT_servant,\nGIOPRecvBuffer *_ORBIT_recv_buffer,\nCORBA_Environment *ev,\n", id, id2);
   free(id); free(id2);
 
   cbe_skel_do_op_dcl_shared(ski);
@@ -891,7 +1025,7 @@ cbe_skel_do_inherited_op_dcl(CBESkelInfo *ski, IDL_tree current_interface)
     g_error("[%s_%s] You cannot have a return value from a oneway operation!", id, IDL_IDENT(ident).str);
   }
 
-  fprintf(ski->of, "\nvoid\nskel_%s_%s(POA_%s *_ORBIT_servant,\nGIOPRecvBuffer *_ORBIT_recv_buffer,\nCORBA_Environment *ev,\n", id, IDL_IDENT(ident).str, id);
+  fprintf(ski->of, "\nstatic void\nskel_%s_%s(POA_%s *_ORBIT_servant,\nGIOPRecvBuffer *_ORBIT_recv_buffer,\nCORBA_Environment *ev,\n", id, IDL_IDENT(ident).str, id);
   free(id);
 
   cbe_skel_do_op_dcl_shared(ski);
@@ -900,11 +1034,13 @@ cbe_skel_do_inherited_op_dcl(CBESkelInfo *ski, IDL_tree current_interface)
 static void
 cbe_skel_op_params_marshal(CBESkelInfo *ski)
 {
-  CBEMarshalInfo mi = {ski->of, NULL, NULL, NULL, NULL};
+  CBEMarshalInfo mi = {NULL, NULL, NULL, NULL, NULL};
   IDL_tree curitem;
   GString *tmpstr;
   int n, i;
 
+  mi.of = ski->of;
+
   if(cbe_op_dcl_nparams(ski->tree, DATA_INOUT|DATA_OUT|DATA_RETURN) <= 0)
     return;
 
@@ -1069,6 +1205,7 @@ cbe_skel_op_param_free(CBESkelInfo *ski)
 	      IDL_IDENT(IDL_PARAM_DCL(ski->tree).simple_declarator).str);
       break;
     default:
+      break;
     }
     break;
   case IDL_PARAM_OUT:
@@ -1126,11 +1263,13 @@ cbe_skel_op_params_free(CBESkelInfo *ski)
 static void
 cbe_skel_op_params_demarshal(CBESkelInfo *ski)
 {
-  CBEDemarshalInfo dmi = {ski->of, NULL, NULL, NULL, NULL, TRUE, FALSE,
+  CBEDemarshalInfo dmi = {NULL, NULL, NULL, NULL, NULL, TRUE, FALSE,
 			  "(((ORBit_ObjectKey *)_ORBIT_servant->_private)->object->orb)"};
   IDL_tree curitem;
   GString *tmpstr = g_string_new(NULL);
 
+  dmi.of = ski->of;
+
   fprintf(ski->of,
 	  "  if(giop_msg_conversion_needed(GIOP_MESSAGE_BUFFER(_ORBIT_recv_buffer))) {\n");
 
@@ -1183,121 +1322,121 @@ cbe_skel_do_string(CBESkelInfo *ski)
 static void
 cbe_skel_do_type_any(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_type_array(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_type_boolean(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_type_char(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_type_dcl(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_type_enum(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_type_fixed(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_type_float(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_type_integer(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_type_object(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_type_octet(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_type_sequence(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_type_string(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_type_struct(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_type_union(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_type_wide_char(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_type_wide_string(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_unaryop(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_wide_char(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 static void
 cbe_skel_do_wide_string(CBESkelInfo *ski)
 {
-  //g_assert(!"Not yet implemented");
+  /*g_assert(!"Not yet implemented");*/
 }
 
 /* Dead code repo */
