@@ -76,7 +76,7 @@ orbit_cbe_write_stubs(FILE *outfile, IDL_ns ns, IDL_tree tree,
 		   " */\n\n");
   fprintf(outfile, "#include <string.h>\n");
   fprintf(outfile, "#include \"%s\"\n\n", header_filename);
-  fprintf(outfile, "#define GET_ATOM(x) ({ GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->decoder(&x, (GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur), sizeof(x)); ((char *)GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur) += sizeof(x); })\n");
+  fprintf(outfile, "#define GET_ATOM(x) G_STMT_START{ GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->decoder(&x, (GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur), sizeof(x)); ((guchar *)GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur) += sizeof(x); }G_STMT_END\n");
 
   orbit_cbe_stub_process_piece(&sti);
 }
@@ -467,6 +467,7 @@ cbe_stub_interface_print_inheritance_ops(IDL_tree node, CBEStubInfo *current)
       cbe_stub_do_inherited_attr_dcl(&substi, current->tree);
       break;
     default:
+      break;
     }
   }
 }
@@ -732,6 +733,7 @@ cbe_print_var_dcl(FILE *of, IDL_tree tree, gboolean for_skels)
 	fprintf(of, "= {0, 0, NULL, CORBA_FALSE}");
 	break;
       default:
+        break;
       }
   }
 }
@@ -747,6 +749,9 @@ cbe_print_param_dcl(FILE *of, IDL_tree tree)
     fprintf(of, " _ORBIT_retval");
   } else {
 
+    if(IDL_PARAM_DCL(tree).attr == IDL_PARAM_IN)
+      fprintf(of, "const ");
+
     orbit_cbe_write_typespec(of,
 			     IDL_PARAM_DCL(tree).param_type_spec);
 
@@ -972,7 +977,9 @@ cbe_stub_op_params_marshal(CBEStubInfo *sti)
   IDL_tree curitem;
   int n, i;
   GString *tmpstr = g_string_new(NULL);
-  CBEMarshalInfo mi = {sti->of, NULL, NULL, NULL, NULL, FALSE};
+  CBEMarshalInfo mi = {NULL, NULL, NULL, NULL, NULL, FALSE};
+
+  mi.of = sti->of;
 
   for(curitem = IDL_OP_DCL(sti->tree).parameter_dcls;
       curitem; curitem = IDL_LIST(curitem).next) {
@@ -1076,6 +1083,7 @@ cbe_stub_op_param_alloc(FILE *of, IDL_tree node, GString *tmpstr)
     if(cbe_type_is_fixed_length(ts))
       return;
   default:
+    break;
   }
 
   g_string_assign(tmpstr, "");
@@ -1132,6 +1140,7 @@ cbe_stub_op_param_free(FILE *of, IDL_tree node, GString *tmpstr)
   case IDLN_TYPE_STRING:
     fprintf(of, "CORBA_free(%s);\n", tmpstr->str + 1);
   default:
+    break;
   }
 }
 
@@ -1167,11 +1176,12 @@ cbe_stub_op_params_alloc(CBEStubInfo *sti)
 static void
 cbe_stub_op_params_demarshal(CBEStubInfo *sti)
 {
-  CBEDemarshalInfo dmi = {sti->of, NULL, NULL, NULL, NULL, TRUE, TRUE, "(_obj->orb)"};
+  CBEDemarshalInfo dmi = {NULL, NULL, NULL, NULL, NULL, TRUE, TRUE, "(_obj->orb)"};
   IDL_tree curitem;
   int n, i;
   GString *tmpstr = g_string_new(NULL);
 
+  dmi.of = sti->of;
   dmi.param = NULL;
 
   fprintf(sti->of, "if(giop_msg_conversion_needed(GIOP_MESSAGE_BUFFER(_ORBIT_recv_buffer))) {\n");
