@@ -90,9 +90,9 @@ cbe_tc_generate(CBETCGenInfo *tci)
   int n;
   char *ctmp;
   CBETCGenInfo subtci;
-  int union_sublabels_array_ctr, union_default_index = -1;
+  int union_sublabels_array_ctr=random_id++, union_default_index = -1;
   int subnames_id = random_id++,
-    subtypes_id=random_id++, sublabels_id=random_id++;
+    subtypes_id = random_id++, sublabels_id = random_id++;
 
   if(IDL_NODE_TYPE(tci->ts) == IDLN_TYPE_DCL) {
     subtci = *tci;
@@ -128,6 +128,9 @@ cbe_tc_generate(CBETCGenInfo *tci)
      && (IDL_list_length(IDL_TYPE_ARRAY(tci->ts).size_list)
 	 > tci->array_gen_ctr)) {
 
+    curitem = IDL_list_nth(IDL_TYPE_ARRAY(tci->ts).size_list,
+			   tci->array_gen_ctr-1);
+
     subtci = *tci;
     subtci.structname = ctmp = cbe_tc_generate_tcstruct_name(curitem);
     subtci.array_gen_ctr++;
@@ -234,6 +237,7 @@ cbe_tc_generate(CBETCGenInfo *tci)
     fprintf(tci->of, "};\n");
     break;
   default:
+    break;
   }
 
   /* subtypes */
@@ -331,6 +335,7 @@ cbe_tc_generate(CBETCGenInfo *tci)
     fprintf(tci->of, "{(CORBA_TypeCode)&%s_struct};\n", tci->substructname);
     break;
   default:
+    break;
   }
 
   /* sublabels */
@@ -353,7 +358,7 @@ cbe_tc_generate(CBETCGenInfo *tci)
 	  curlabel = IDL_LIST(curlabel).next) {
 
 	if(IDL_LIST(curlabel).data) {
-	  fprintf(tci->of, "{&TC_%s, &anon_sublabel_values_array%d[%d], CORBA_FALSE}",
+	  fprintf(tci->of, "{(CORBA_TypeCode)&TC_%s_struct, (gpointer)&anon_sublabel_values_array%d[%d], CORBA_FALSE}",
 		  ctmp, union_sublabels_array_ctr, n);
 	  n++;
 	} else {
@@ -370,10 +375,12 @@ cbe_tc_generate(CBETCGenInfo *tci)
     fprintf(tci->of, "};\n");
     break;
   default:
+    break;
   }
 
   fprintf(tci->of, "const struct CORBA_TypeCode_struct %s_struct = {\n",
 	  tci->structname);
+
   /* pseudoobject stuff */
   fprintf(tci->of, "{ { (ORBit_RootObject_Interface*)&ORBit_TypeCode_epv, TRUE, -1 }, ORBIT_PSEUDO_TYPECODE },\n");
 
@@ -490,8 +497,10 @@ cbe_tc_generate(CBETCGenInfo *tci)
     }
   default:
     fprintf(tci->of, "NULL");
+#if 0
     g_message("Type %s needs no name",
 	      IDL_tree_type_names[IDL_NODE_TYPE(tci->ts)]);
+#endif
   }
 
   fprintf(tci->of, ",");
@@ -674,7 +683,7 @@ cbe_tc_generate(CBETCGenInfo *tci)
   switch(IDL_NODE_TYPE(tci->ts)) {
   case IDLN_TYPE_UNION:
     ctmp = orbit_cbe_get_typename(IDL_TYPE_UNION(tci->ts).switch_type_spec);
-    fprintf(tci->of, "&TC_%s_struct", ctmp);
+    fprintf(tci->of, "(CORBA_TypeCode)&TC_%s_struct", ctmp);
     g_free(ctmp);
     break;
   default:
@@ -702,6 +711,8 @@ cbe_tc_generate(CBETCGenInfo *tci)
     fprintf(tci->of, "0, 0");
 
   fprintf(tci->of, "\n};\n");
+#if 0
   fprintf(tci->of, "const CORBA_TypeCode %s = (const CORBA_TypeCode) &%s_struct;\n",
 	  tci->structname, tci->structname);
+#endif
 }
