@@ -39,28 +39,20 @@ static CORBA_boolean CDR_buffer_grow(CDR_Codec *codec, const unsigned int growth
 	unsigned int real_growth;
 	div_t divvy;
 
-	g_assert(codec!=NULL);
-
-	divvy=div(growth, CDR_GROW_AMOUNT);
-	real_growth=CDR_GROW_AMOUNT * (divvy.quot+1);
-
-	ORBit_Trace(TraceMod_CDR, TraceLevel_Debug, "CDR_buffer_grow: growing buffer by %d (%d) to %d\n", real_growth, growth, codec->buf_len+CDR_GROW_AMOUNT);
-
-	codec->buffer=(CORBA_octet *)g_realloc(codec->buffer, codec->buf_len+real_growth);
-	if(codec->buffer==NULL) {
-		ORBit_Trace(TraceMod_CDR, TraceLevel_Error, "CDR_buffer_grow: realloc error growing buffer\n");
-		return(CORBA_FALSE);
+	if(codec->release_buffer) {
+		divvy=div(growth, CDR_GROW_AMOUNT);
+		real_growth=CDR_GROW_AMOUNT * (divvy.quot+1);
+		
+		codec->buffer=(CORBA_octet *)g_realloc(codec->buffer,
+						       codec->buf_len
+						       +real_growth);
 	}
 
-	return(CORBA_TRUE);
+	return CORBA_TRUE;
 }
 
 static void CDR_buffer_puts(CDR_Codec *codec, const void *data, const unsigned int len)
 {
-	g_assert(codec!=NULL);
-	g_assert(codec->readonly!=CORBA_TRUE);
-	g_assert(codec->wptr<=codec->buf_len);
-
 	if(codec->wptr+len > codec->buf_len) {
 		CORBA_boolean res=CDR_buffer_grow(codec, len);
 
@@ -76,10 +68,6 @@ static void CDR_buffer_puts(CDR_Codec *codec, const void *data, const unsigned i
 
 CORBA_boolean CDR_buffer_gets(CDR_Codec *codec, void *dest, const unsigned int len)
 {
-	g_assert(codec!=NULL);
-	g_assert(dest!=NULL);
-	g_assert(codec->rptr<=codec->buf_len);
-
 	if(codec->rptr+len > codec->buf_len) {
 		ORBit_Trace(TraceMod_CDR, TraceLevel_Debug, "CDR_buffer_gets: attempt to read past end of buffer\n");
 		return(CORBA_FALSE);
@@ -93,10 +81,6 @@ CORBA_boolean CDR_buffer_gets(CDR_Codec *codec, void *dest, const unsigned int l
 
 static void CDR_buffer_put(CDR_Codec *codec, void *datum)
 {
-	g_assert(codec!=NULL);
-	g_assert(codec->readonly!=CORBA_TRUE);
-	g_assert(codec->wptr<=codec->buf_len);
-
 	if(codec->wptr+1 > codec->buf_len) {
 		CORBA_boolean res=CDR_buffer_grow(codec, 1);
 
@@ -111,10 +95,6 @@ static void CDR_buffer_put(CDR_Codec *codec, void *datum)
 
 static CORBA_boolean CDR_buffer_get(CDR_Codec *codec, void *dest)
 {
-	g_assert(codec!=NULL);
-	g_assert(dest!=NULL);
-	g_assert(codec->rptr<=codec->buf_len);
-
 	if(codec->rptr+1 > codec->buf_len) {
 		ORBit_Trace(TraceMod_CDR, TraceLevel_Debug, "CDR_buffer_get: attempt to read past end of buffer\n");
 		return(CORBA_FALSE);
@@ -124,6 +104,7 @@ static CORBA_boolean CDR_buffer_get(CDR_Codec *codec, void *dest)
 	return(CORBA_TRUE);
 }
 
+#ifdef lame_slow_code
 static void CDR_buffer_put2(CDR_Codec *codec, void *datum)
 {
 	unsigned long align;
@@ -371,7 +352,49 @@ static CORBA_boolean CDR_buffer_get16(CDR_Codec *codec, void *dest)
 	return(CORBA_TRUE);
 }
 #endif
+#endif /* lame_slow_code */
 
+#define CDR_buffer_put2(codec, datum) CDR_buffer_putn(codec, datum, 2)
+#define CDR_buffer_put4(codec, datum) CDR_buffer_putn(codec, datum, 4)
+#define CDR_buffer_put8(codec, datum) CDR_buffer_putn(codec, datum, 8)
+#define CDR_buffer_put16(codec, datum) CDR_buffer_putn(codec, datum, 16)
+#define CDR_buffer_get2(codec, dest) CDR_buffer_getn(codec, dest, 2)
+#define CDR_buffer_get4(codec, dest) CDR_buffer_getn(codec, dest, 4)
+#define CDR_buffer_get8(codec, dest) CDR_buffer_getn(codec, dest, 8)
+#define CDR_buffer_get16(codec, dest) CDR_buffer_getn(codec, dest, 16)
+
+static CORBA_boolean
+CDR_buffer_getn(CDR_Codec *codec, void *dest, int bsize)
+{
+	codec->rptr = (unsigned long)ALIGN_ADDRESS(codec->rptr, bsize);
+	if(codec->host_endian==codec->data_endian)
+		memcpy(dest, codec->buffer + codec->rptr, bsize);
+	else
+		iiop_byteswap(dest, codec->buffer + codec->rptr, bsize);
+	codec->rptr += bsize;
+
+	return CORBA_TRUE;
+}
+
+static CORBA_boolean
+CDR_buffer_putn(CDR_Codec *codec, void *datum, int bsize)
+{
+	codec->wptr = (unsigned long)ALIGN_ADDRESS(codec->wptr, bsize);
+	if(codec->host_endian==codec->data_endian)
+		memcpy(codec->buffer + codec->wptr, datum, bsize);
+	else
+		iiop_byteswap(codec->buffer + codec->wptr, datum, bsize);
+	codec->wptr += bsize;
+
+	return CORBA_TRUE;
+}
+
+#define CDR_swap2(d,s) iiop_byteswap((d), (s), 2)
+#define CDR_swap4(d,s) iiop_byteswap((d), (s), 4)
+#define CDR_swap8(d,s) iiop_byteswap((d), (s), 8)
+#define CDR_swap16(d,s) iiop_byteswap((d), (s), 16)
+
+#ifdef lame_slow_code
 static void CDR_swap2(void *d, void *s)
 {
 	((CORBA_octet *)d)[0]=((CORBA_octet *)s)[1];
@@ -417,257 +440,131 @@ static void CDR_swap16(void *d, void *s)
 	((CORBA_octet *)d)[14]=((CORBA_octet *)s)[1];
 	((CORBA_octet *)d)[15]=((CORBA_octet *)s)[0];
 }
+#endif
+
 
 void CDR_put_short(CDR_Codec *codec, CORBA_short s)
 {
-	g_assert(codec!=NULL);
-
-	if(codec->host_endian==codec->data_endian) {
-		CDR_buffer_put2(codec, &s);
-	} else {
-		CORBA_short s2;
-
-		CDR_swap2(&s2, &s);
-
-		CDR_buffer_put2(codec, &s2);
-	}
+	CDR_buffer_put2(codec, &s);
 }
 
 void CDR_put_ushort(CDR_Codec *codec, CORBA_unsigned_short us)
 {
-	g_assert(codec!=NULL);
-
-	if(codec->host_endian==codec->data_endian) {
-		CDR_buffer_put2(codec, &us);
-	} else {
-		CORBA_unsigned_short us2;
-
-		CDR_swap2(&us2, &us);
-
-		CDR_buffer_put2(codec, &us2);
-	}
+	CDR_buffer_put2(codec, &us);
 }
 
 CORBA_boolean CDR_get_ushort(CDR_Codec *codec, CORBA_unsigned_short *us)
 {
-	g_assert(codec!=NULL);
-	g_assert(us!=NULL);
-
-	if(codec->host_endian==codec->data_endian) {
-		return(CDR_buffer_get2(codec, us));
-	} else {
-		CORBA_unsigned_short us2;
-
-		if(CDR_buffer_get2(codec, &us2)==CORBA_FALSE) {
-			return(CORBA_FALSE);
-		}
-
-		CDR_swap2(us, &us2);
-
-		return(CORBA_TRUE);
-	}
+	return CDR_buffer_get2(codec, us);
 }
 
 void CDR_put_long(CDR_Codec *codec, CORBA_long l)
 {
-	g_assert(codec!=NULL);
-
-	if(codec->host_endian==codec->data_endian) {
-		CDR_buffer_put4(codec, &l);
-	} else {
-		CORBA_long l2;
-
-		CDR_swap4(&l2, &l);
-
-		CDR_buffer_put4(codec, &l2);
-	}
+	CDR_buffer_put4(codec, &l);
 }
 
 void CDR_put_ulong(CDR_Codec *codec, CORBA_unsigned_long ul)
 {
-	g_assert(codec!=NULL);
-
-	if(codec->host_endian==codec->data_endian) {
-		CDR_buffer_put4(codec, &ul);
-	} else {
-		CORBA_unsigned_long ul2;
-
-		CDR_swap4(&ul2, &ul);
-
-		CDR_buffer_put4(codec, &ul2);
-	}
+	CDR_buffer_put4(codec, &ul);
 }
 
 CORBA_boolean CDR_get_ulong(CDR_Codec *codec, CORBA_unsigned_long *ul)
 {
-	g_assert(codec!=NULL);
-	g_assert(ul!=NULL);
-
-	if(codec->host_endian==codec->data_endian) {
-		return(CDR_buffer_get4(codec, ul));
-	} else {
-		CORBA_unsigned_long ul2;
-
-		if(CDR_buffer_get4(codec, &ul2)==CORBA_FALSE) {
-			return(CORBA_FALSE);
-		}
-
-		CDR_swap4(ul, &ul2);
-
-		return(CORBA_TRUE);
-	}
+	return CDR_buffer_get4(codec, ul);
 }
 
 #ifdef HAVE_CORBA_LONG_LONG
 void CDR_put_long_long(CDR_Codec *codec, CORBA_long_long ll)
 {
-	g_assert(codec!=NULL);
-
-	if(codec->host_endian==codec->data_endian) {
-		CDR_buffer_put8(codec, &ll);
-	} else {
-		CORBA_long_long ll2;
-
-		CDR_swap8(&ll2, &ll);
-
-		CDR_buffer_put8(codec, &ll2);
-	}
+	CDR_buffer_put8(codec, &ll);
 }
 
 void CDR_put_ulong_long(CDR_Codec *codec, CORBA_unsigned_long_long ull)
 {
-	g_assert(codec!=NULL);
-
-	if(codec->host_endian==codec->data_endian) {
-		CDR_buffer_put8(codec, &ull);
-	} else {
-		CORBA_unsigned_long_long ull2;
-
-		CDR_swap8(&ull2, &ull);
-
-		CDR_buffer_put8(codec, &ull2);
-	}
+	CDR_buffer_put8(codec, &ull);
 }
 #endif
 
 void CDR_put_float(CDR_Codec *codec, CORBA_float f)
 {
-	g_assert(codec!=NULL);
-
-	if(codec->host_endian==codec->data_endian) {
-		CDR_buffer_put4(codec, &f);
-	} else {
-		CORBA_float f2;
-
-		CDR_swap4(&f2, &f);
-
-		CDR_buffer_put4(codec, &f2);
-	}
+	CDR_buffer_put4(codec, &f);
 }
 
 void CDR_put_double(CDR_Codec *codec, CORBA_double d)
 {
-	g_assert(codec!=NULL);
-
-	if(codec->host_endian==codec->data_endian) {
-		CDR_buffer_put8(codec, &d);
-	} else {
-		CORBA_double d2;
-
-		CDR_swap8(&d2, &d);
-
-		CDR_buffer_put8(codec, &d2);
-	}
+	CDR_buffer_put8(codec, &d);
 }
 
 void CDR_put_long_double(CDR_Codec *codec, CORBA_long_double ld)
 {
-	g_assert(codec!=NULL);
-
-	if(codec->host_endian==codec->data_endian) {
-		CDR_buffer_put16(codec, &ld);
-	} else {
-		CORBA_long_double ld2;
-
-		CDR_swap16(&ld2, &ld);
-
-		CDR_buffer_put16(codec, &ld2);
-	}
+	CDR_buffer_put16(codec, &ld);
 }
 
 void CDR_put_octet(CDR_Codec *codec, CORBA_octet datum)
 {
-	g_assert(codec!=NULL);
-
 	CDR_buffer_put(codec, &datum);
 }
 
 CORBA_boolean CDR_get_octet(CDR_Codec *codec, CORBA_octet *datum)
 {
-	g_assert(codec!=NULL);
-	g_assert(datum!=NULL);
-
 	return(CDR_buffer_get(codec, datum));
 }
 
 void CDR_put_octets(CDR_Codec *codec, void *data, unsigned long len)
 {
-	g_assert(codec!=NULL);
-
 	CDR_buffer_puts(codec, data, len);
 }
 
 void CDR_put_char(CDR_Codec *codec, CORBA_char c)
 {
-	g_assert(codec!=NULL);
-
 	CDR_buffer_put(codec, &c);
 }
 
 void CDR_put_boolean(CDR_Codec *codec, CORBA_boolean datum)
 {
-	g_assert(codec!=NULL);
-
-	if(datum==CORBA_TRUE) {
-		CDR_buffer_put(codec, (void *)1);
-	} else {
-		CDR_buffer_put(codec, (void *)0);
-	}
+	datum = datum&&1;
+	CDR_buffer_put(codec, &datum);
 }
 
 void CDR_put_string(CDR_Codec *codec, const char *str)
 {
 	unsigned int len;
 
-	g_assert(codec!=NULL);
-	g_assert(str!=NULL);
-
 	len=strlen(str)+1;
 
 	CDR_put_ulong(codec, len);
 	CDR_buffer_puts(codec, str, len);
 }
 
+CORBA_boolean CDR_get_string_static(CDR_Codec *codec,
+				    CORBA_char **str)
+{
+	CORBA_unsigned_long len;
+
+	if(CDR_get_ulong(codec, &len)==CORBA_FALSE)
+		return CORBA_FALSE;
+
+	if((codec->rptr + len) > codec->buf_len)
+		return CORBA_FALSE;
+
+	*str = codec->buffer + codec->rptr;
+
+	codec->rptr += len;
+
+	return CORBA_TRUE;
+}
+
 CORBA_boolean CDR_get_string(CDR_Codec *codec, CORBA_char **str)
 {
 	CORBA_unsigned_long len;
 
-	g_assert(codec!=NULL);
-	g_assert(str!=NULL);
-
-	if(CDR_get_ulong(codec, &len)==CORBA_FALSE) {
+	if(CDR_get_ulong(codec, &len)==CORBA_FALSE)
 		return(CORBA_FALSE);
-	}
 
-	if(len==0) {
+	if(len==0)
 		return(CORBA_FALSE);
-	}
 
-	*str=g_new0(CORBA_char, len);
-	if(*str==NULL) {
-		ORBit_Trace(TraceMod_CDR, TraceLevel_Error, "CDR_get_string: malloc error creating string\n");
-		return(CORBA_FALSE);
-	}
+	*str=g_new(CORBA_char, len);
 
 	if(CDR_buffer_gets(codec, *str, len)==CORBA_FALSE) {
 		g_free(*str);
@@ -687,32 +584,28 @@ CORBA_boolean CDR_get_seq_begin(CDR_Codec *codec, CORBA_unsigned_long *ul)
 	return(CDR_get_ulong(codec, ul));
 }
 
+CDR_Codec *CDR_codec_init_static(CDR_Codec *codec)
+{
+	memset(codec, 0, sizeof(CDR_Codec));
+
+	codec->host_endian = FLAG_ENDIANNESS;
+
+	return codec;
+}
+
 CDR_Codec *CDR_codec_init(void)
 {
 	CDR_Codec *new;
 
 	new=g_new0(CDR_Codec, 1);
+	CDR_codec_init_static(new);
 	new->release_buffer = CORBA_TRUE;
-	new->host_endian = FLAG_ENDIANNESS;
-
-	if(new==NULL) {
-		ORBit_Trace(TraceMod_CDR, TraceLevel_Error, "CDR_codec_init: malloc error creating codec\n");
-		return(NULL);
-	}
-
-#ifdef WORDS_BIGENDIAN
-	new->host_endian=BigEndian;
-#else
-	new->host_endian=LittleEndian;
-#endif
 
 	return(new);
 }
 
 void CDR_codec_free(CDR_Codec *codec)
 {
-	g_assert(codec!=NULL);
-
 	if(codec->release_buffer)
 		g_free(codec->buffer);
 
