@@ -111,40 +111,40 @@ ORBit_marshal_value(GIOPSendBuffer *buf,
     case CORBA_tk_short:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_SHORT);
 	giop_message_buffer_append_mem_a(GIOP_MESSAGE_BUFFER(buf), *val, sizeof(CORBA_short));
-	*val += sizeof(CORBA_short);
+	(guchar *)*val += sizeof(CORBA_short);
 	break;
     case CORBA_tk_enum:
     case CORBA_tk_long:
     case CORBA_tk_ulong:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_LONG);
 	giop_message_buffer_append_mem_a(GIOP_MESSAGE_BUFFER(buf), *val, sizeof(CORBA_long));
-	*val += sizeof(CORBA_long);
+	(guchar *)*val += sizeof(CORBA_long);
 	break;
     case CORBA_tk_float:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_FLOAT);
 	giop_message_buffer_append_mem_a(GIOP_MESSAGE_BUFFER(buf), *val, sizeof(CORBA_float));
-	*val += sizeof(CORBA_float);
+	(guchar *)*val += sizeof(CORBA_float);
 	break;
     case CORBA_tk_double:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_DOUBLE);
 	giop_message_buffer_append_mem_a(GIOP_MESSAGE_BUFFER(buf), *val, sizeof(CORBA_double));
-	*val += sizeof(CORBA_double);
+	(guchar *)*val += sizeof(CORBA_double);
 	break;
     case CORBA_tk_boolean:
     case CORBA_tk_char:
     case CORBA_tk_octet:
 	giop_message_buffer_append_mem(GIOP_MESSAGE_BUFFER(buf), *val, sizeof(CORBA_octet));
-	*val += sizeof(CORBA_octet);
+	(guchar *)*val += sizeof(CORBA_octet);
 	break;
     case CORBA_tk_any:
 	*val = ALIGN_ADDRESS(*val, MAX(ALIGNOF_CORBA_STRUCT, ALIGNOF_CORBA_POINTER));
 	ORBit_marshal_any(buf, *val);
-	*val += sizeof(CORBA_any);
+	(guchar *)*val += sizeof(CORBA_any);
 	break;
     case CORBA_tk_TypeCode:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_POINTER);
 	ORBit_encode_CORBA_TypeCode(*val, buf);
-	*val += sizeof(CORBA_TypeCode);
+	(guchar *)*val += sizeof(CORBA_TypeCode);
 	break;
     case CORBA_tk_Principal:
 	*val = ALIGN_ADDRESS(*val,
@@ -155,14 +155,14 @@ ORBit_marshal_value(GIOPSendBuffer *buf,
 	giop_message_buffer_append_mem_a(GIOP_MESSAGE_BUFFER(buf), *val, sizeof(CORBA_unsigned_long));
 
 	giop_message_buffer_append_mem(GIOP_MESSAGE_BUFFER(buf),
-				       *(char**)(*val+sizeof(CORBA_unsigned_long)),
+				       *(char**)((char *)*val+sizeof(CORBA_unsigned_long)),
 				       ulval);
-	*val += sizeof(CORBA_Principal);
+	(guchar *)*val += sizeof(CORBA_Principal);
 	break;
     case CORBA_tk_objref:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_POINTER);
 	ORBit_marshal_object(buf, *val);
-	*val += sizeof(CORBA_Object);
+	(guchar *)*val += sizeof(CORBA_Object);
 	break;
     case CORBA_tk_except:
     case CORBA_tk_struct:
@@ -194,7 +194,7 @@ ORBit_marshal_value(GIOPSendBuffer *buf,
 					       sizeof(CORBA_unsigned_long));
 	giop_message_buffer_append_mem(GIOP_MESSAGE_BUFFER(buf), *(char **)*val, ulval);
 	
-	*val += sizeof(char *);
+	(guchar *)*val += sizeof(char *);
 	break;
     case CORBA_tk_string:
 	ulval = strlen(*(char **)*val) + 1;
@@ -206,7 +206,7 @@ ORBit_marshal_value(GIOPSendBuffer *buf,
 					       sizeof(CORBA_unsigned_long));
 	giop_message_buffer_append_mem(GIOP_MESSAGE_BUFFER(buf), *(char **)*val, ulval);
 	
-	*val += sizeof(char *);
+	(guchar *)*val += sizeof(char *);
 	break;
     case CORBA_tk_sequence:
 	{
@@ -224,7 +224,7 @@ ORBit_marshal_value(GIOPSendBuffer *buf,
 	    for(i = 0; i < sval->_length; i++)
 		ORBit_marshal_value(buf, &subval, tc->subtypes[0], mi);
 	    
-	    *val += sizeof(CORBA_sequence_octet);
+	    (guchar *)*val += sizeof(CORBA_sequence_octet);
 	}
 	break;
     case CORBA_tk_array:
@@ -335,7 +335,7 @@ ORBit_marshal_arg(GIOPSendBuffer *buf,
 
 
 void
-ORBit_marshal_any(GIOPSendBuffer *buf, CORBA_any *val)
+ORBit_marshal_any(GIOPSendBuffer *buf, const CORBA_any *val)
 {
     ORBit_marshal_value_info mi;
 
@@ -349,7 +349,7 @@ ORBit_marshal_any(GIOPSendBuffer *buf, CORBA_any *val)
 size_t
 ORBit_gather_alloc_info(CORBA_TypeCode tc)
 {
-    int i, n, align, prevalign, sum, prev;
+    int i, n, align=1, prevalign, sum, prev;
     size_t block_size;
 
     switch(tc->kind) {
@@ -398,6 +398,7 @@ ORBit_gather_alloc_info(CORBA_TypeCode tc)
     case CORBA_tk_union:
 	sum = ORBit_gather_alloc_info(tc->discriminator);
 	n = -1;
+	align = 1;
 	for(prev = prevalign = i = 0; i < tc->sub_parts; i++) {
 	    prevalign = align;
 	    align = ORBit_find_alignment(tc->subtypes[i]);
@@ -454,12 +455,12 @@ ORBit_demarshal_allocate_mem(CORBA_TypeCode tc, gint nelements)
 			   (ORBit_free_childvals)ORBit_free_via_TypeCode,
 			   GINT_TO_POINTER(nelements), sizeof(CORBA_TypeCode));
 
-    *(CORBA_TypeCode *)(retval-sizeof(ORBit_mem_info)-sizeof(CORBA_TypeCode)) = (CORBA_TypeCode)CORBA_Object_duplicate((CORBA_Object)tc, NULL);
+    *(CORBA_TypeCode *)((char *)retval-sizeof(ORBit_mem_info)-sizeof(CORBA_TypeCode)) = (CORBA_TypeCode)CORBA_Object_duplicate((CORBA_Object)tc, NULL);
 
     return retval;
 }
 
-#define DM_GET_ATOM(x, n) ({ GIOP_RECV_BUFFER(buf)->decoder(x, (GIOP_RECV_BUFFER(buf)->cur), n); ((char *)GIOP_RECV_BUFFER(buf)->cur) += n; })
+#define DM_GET_ATOM(x, n) G_STMT_START{ GIOP_RECV_BUFFER(buf)->decoder(x, (GIOP_RECV_BUFFER(buf)->cur), n); ((char *)GIOP_RECV_BUFFER(buf)->cur) += n; }G_STMT_END
 
 static void
 ORBit_demarshal_value(GIOPRecvBuffer *buf,
@@ -482,7 +483,7 @@ ORBit_demarshal_value(GIOPRecvBuffer *buf,
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_SHORT);
 	buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(CORBA_short));
 	DM_GET_ATOM(*val, sizeof(CORBA_short));
-	*val += sizeof(CORBA_short);
+	(guchar *)*val += sizeof(CORBA_short);
 	break;
     case CORBA_tk_long:
     case CORBA_tk_ulong:
@@ -490,38 +491,38 @@ ORBit_demarshal_value(GIOPRecvBuffer *buf,
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_LONG);
 	buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(CORBA_long));
 	DM_GET_ATOM(*val, sizeof(CORBA_long));
-	*val += sizeof(CORBA_long);
+	(guchar *)*val += sizeof(CORBA_long);
 	break;
     case CORBA_tk_longlong:
     case CORBA_tk_ulonglong:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_LONG_LONG);
 	buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(CORBA_long_long));
 	DM_GET_ATOM(*val, sizeof(CORBA_long_long));
-	*val += sizeof(CORBA_long_long);
+	(guchar *)*val += sizeof(CORBA_long_long);
 	break;
     case CORBA_tk_longdouble:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_LONG_DOUBLE);
 	buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(CORBA_long_double));
 	DM_GET_ATOM(*val, sizeof(CORBA_long_double));
-	*val += sizeof(CORBA_long_double);
+	(guchar *)*val += sizeof(CORBA_long_double);
 	break;
     case CORBA_tk_float:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_FLOAT);
 	buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(CORBA_float));
 	DM_GET_ATOM(*val, sizeof(CORBA_float));
-	*val += sizeof(CORBA_float);
+	(guchar *)*val += sizeof(CORBA_float);
 	break;
     case CORBA_tk_double:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_DOUBLE);
 	buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(CORBA_double));
 	DM_GET_ATOM(*val, sizeof(CORBA_double));
-	*val += sizeof(CORBA_double);
+	(guchar *)*val += sizeof(CORBA_double);
 	break;
     case CORBA_tk_boolean:
     case CORBA_tk_char:
     case CORBA_tk_octet:
 	DM_GET_ATOM(*val, sizeof(CORBA_octet));
-	*val += sizeof(CORBA_octet);
+	(guchar *)*val += sizeof(CORBA_octet);
 	break;
     case CORBA_tk_any:
 	{
@@ -533,14 +534,14 @@ ORBit_demarshal_value(GIOPRecvBuffer *buf,
 	    decoded = *val;
 	    decoded->_release = CORBA_FALSE;
 	    ORBit_demarshal_any(buf, decoded, dup_strings, orb);
-	    *val += sizeof(CORBA_any);
+	    (guchar *)*val += sizeof(CORBA_any);
 	}
 	break;
     case CORBA_tk_TypeCode:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_POINTER);
 	ORBit_decode_CORBA_TypeCode(*val, buf);
 	CORBA_Object_duplicate(*(CORBA_Object *)*val, NULL);
-	*val += sizeof(CORBA_TypeCode);
+	(guchar *)*val += sizeof(CORBA_TypeCode);
 	break;
     case CORBA_tk_Principal:
 	{
@@ -555,14 +556,14 @@ ORBit_demarshal_value(GIOPRecvBuffer *buf,
 	    DM_GET_ATOM(&p->_length, sizeof(CORBA_long));
 	    p->_buffer = ORBit_alloc(p->_length, NULL, GINT_TO_POINTER(1));
 	    memcpy(p->_buffer, buf->cur, p->_length);
-	    buf->cur += p->_length;
-	    *val += sizeof(CORBA_sequence_octet);
+	    (char *)buf->cur += p->_length;
+	    (guchar *)*val += sizeof(CORBA_sequence_octet);
 	}
 	break;
     case CORBA_tk_objref:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_POINTER);
 	*(CORBA_Object *)*val = ORBit_demarshal_object(buf, orb);
-	*val += sizeof(CORBA_Object);
+	(guchar *)*val += sizeof(CORBA_Object);
 	break;
     case CORBA_tk_except:
     case CORBA_tk_struct:
@@ -596,7 +597,7 @@ ORBit_demarshal_value(GIOPRecvBuffer *buf,
 	    *(char **)*val = CORBA_string_dup(buf->cur);
 	else
 	    *(char **)*val = buf->cur;
-	*val += sizeof(CORBA_char *);
+	(guchar *)*val += sizeof(CORBA_char *);
 	break;
     case CORBA_tk_sequence:
 	{
@@ -615,7 +616,7 @@ ORBit_demarshal_value(GIOPRecvBuffer *buf,
 		   all atoms... */
 		p->_buffer = ORBit_alloc(p->_length, NULL, GINT_TO_POINTER(1));
 		memcpy(p->_buffer, buf->cur, p->_length);
-		buf->cur += p->_length;
+		(char *)buf->cur += p->_length;
 	    } else {
 		p->_buffer = ORBit_demarshal_allocate_mem(tc->subtypes[0],
 							  p->_length);
@@ -628,7 +629,7 @@ ORBit_demarshal_value(GIOPRecvBuffer *buf,
 					  orb);
 	    }
 
-	    *val += sizeof(CORBA_sequence_octet);
+	    (guchar *)*val += sizeof(CORBA_sequence_octet);
 	}
 	break;
     case CORBA_tk_array:
@@ -642,6 +643,7 @@ ORBit_demarshal_value(GIOPRecvBuffer *buf,
 	g_error("CORBA_fixed NYI");
 	break;
     default:
+        break;
     }
 }
 
@@ -692,8 +694,8 @@ _ORBit_copy_value(gpointer *val, gpointer *newval, CORBA_TypeCode tc)
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_SHORT);
 	*newval = ALIGN_ADDRESS(*newval, ALIGNOF_CORBA_SHORT);
 	*(CORBA_short *)*newval = *(CORBA_short *)*val;
-	*val += sizeof(CORBA_short);
-	*newval += sizeof(CORBA_short);
+	(guchar *)*val += sizeof(CORBA_short);
+	(guchar *)*newval += sizeof(CORBA_short);
 	break;
     case CORBA_tk_enum:
     case CORBA_tk_long:
@@ -701,44 +703,44 @@ _ORBit_copy_value(gpointer *val, gpointer *newval, CORBA_TypeCode tc)
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_LONG);
 	*newval = ALIGN_ADDRESS(*newval, ALIGNOF_CORBA_LONG);
 	*(CORBA_long *)*newval = *(CORBA_long *)*val;
-	*val += sizeof(CORBA_long);
-	*newval += sizeof(CORBA_long);
+	(guchar *)*val += sizeof(CORBA_long);
+	(guchar *)*newval += sizeof(CORBA_long);
 	break;
     case CORBA_tk_longlong:
     case CORBA_tk_ulonglong:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_LONG_LONG);
 	*newval = ALIGN_ADDRESS(*newval, ALIGNOF_CORBA_LONG_LONG);
 	*(CORBA_long_long *)*newval = *(CORBA_long_long *)*val;
-	*val += sizeof(CORBA_long_long);
-	*newval += sizeof(CORBA_long_long);
+	(guchar *)*val += sizeof(CORBA_long_long);
+	(guchar *)*newval += sizeof(CORBA_long_long);
 	break;
     case CORBA_tk_longdouble:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_LONG_DOUBLE);
 	*newval = ALIGN_ADDRESS(*newval, ALIGNOF_CORBA_LONG_DOUBLE);
 	*(CORBA_long_double *)*newval = *(CORBA_long_double *)*val;
-	*val += sizeof(CORBA_long_double);
-	*newval += sizeof(CORBA_long_double);
+	(guchar *)*val += sizeof(CORBA_long_double);
+	(guchar *)*newval += sizeof(CORBA_long_double);
 	break;
     case CORBA_tk_float:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_FLOAT);
 	*newval = ALIGN_ADDRESS(*newval, ALIGNOF_CORBA_FLOAT);
 	*(CORBA_long *)*newval = *(CORBA_long *)*val;
-	*val += sizeof(CORBA_float);
-	*newval += sizeof(CORBA_float);
+	(guchar *)*val += sizeof(CORBA_float);
+	(guchar *)*newval += sizeof(CORBA_float);
 	break;
     case CORBA_tk_double:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_DOUBLE);
 	*newval = ALIGN_ADDRESS(*newval, ALIGNOF_CORBA_DOUBLE);
 	*(CORBA_double *)*newval = *(CORBA_double *)*val;
-	*val += sizeof(CORBA_double);
-	*newval += sizeof(CORBA_double);
+	(guchar *)*val += sizeof(CORBA_double);
+	(guchar *)*newval += sizeof(CORBA_double);
 	break;
     case CORBA_tk_boolean:
     case CORBA_tk_char:
     case CORBA_tk_octet:
 	*(CORBA_octet *)*newval = *(CORBA_octet *)*val;
-	*val += sizeof(CORBA_octet);
-	*newval += sizeof(CORBA_octet);
+	(guchar *)*val += sizeof(CORBA_octet);
+	(guchar *)*newval += sizeof(CORBA_octet);
 	break;
     case CORBA_tk_any:
 	{
@@ -752,8 +754,8 @@ _ORBit_copy_value(gpointer *val, gpointer *newval, CORBA_TypeCode tc)
                == FALSE? */
 	    newany->_value = ORBit_copy_value(oldany->_value, oldany->_type);
 	    newany->_release = CORBA_TRUE;
-	    *val += sizeof(CORBA_any);
-	    *newval += sizeof(CORBA_any);
+	    (guchar *)*val += sizeof(CORBA_any);
+	    (guchar *)*newval += sizeof(CORBA_any);
 	}
 	break;
     case CORBA_tk_Principal:
@@ -771,8 +773,8 @@ _ORBit_copy_value(gpointer *val, gpointer *newval, CORBA_TypeCode tc)
 	memcpy(((CORBA_Principal *)*newval)->_buffer,
 	       ((CORBA_Principal *)*val)->_buffer,
 	       ((CORBA_Principal *)*val)->_length);
-	*val += sizeof(CORBA_Principal);
-	*newval += sizeof(CORBA_Principal);
+	(guchar *)*val += sizeof(CORBA_Principal);
+	(guchar *)*newval += sizeof(CORBA_Principal);
 	break;
     case CORBA_tk_TypeCode:
     case CORBA_tk_objref:
@@ -780,8 +782,8 @@ _ORBit_copy_value(gpointer *val, gpointer *newval, CORBA_TypeCode tc)
 	*newval = ALIGN_ADDRESS(*newval, ALIGNOF_CORBA_POINTER);
 	*(CORBA_Object *)*newval = CORBA_Object_duplicate(*(CORBA_Object *)*val,
 							  NULL);
-	*val += sizeof(CORBA_Object);
-	*newval += sizeof(CORBA_Object);
+	(guchar *)*val += sizeof(CORBA_Object);
+	(guchar *)*newval += sizeof(CORBA_Object);
 	break;
     case CORBA_tk_struct:
     case CORBA_tk_except:
@@ -810,8 +812,8 @@ _ORBit_copy_value(gpointer *val, gpointer *newval, CORBA_TypeCode tc)
 	*newval = ALIGN_ADDRESS(*newval, ALIGNOF_CORBA_POINTER);
 	
 	*(CORBA_char **)*newval = CORBA_string_dup(*(CORBA_char **)*val);
-	*val += sizeof(CORBA_char *);
-	*newval += sizeof(CORBA_char *);
+	(guchar *)*val += sizeof(CORBA_char *);
+	(guchar *)*newval += sizeof(CORBA_char *);
 	break;
     case CORBA_tk_sequence:
 	*val = ALIGN_ADDRESS(*val,
@@ -832,8 +834,8 @@ _ORBit_copy_value(gpointer *val, gpointer *newval, CORBA_TypeCode tc)
 	for(i = 0; i < ((CORBA_Principal *)*newval)->_length; i++) {
 	    _ORBit_copy_value(&pval1, &pval2, tc->subtypes[0]);
 	}
-	*val += sizeof(CORBA_sequence_octet);
-	*newval += sizeof(CORBA_sequence_octet);
+	(guchar *)*val += sizeof(CORBA_sequence_octet);
+	(guchar *)*newval += sizeof(CORBA_sequence_octet);
 	break;
     case CORBA_tk_array:
 	for(i = 0; i < tc->length; i++) {
