@@ -32,9 +32,13 @@
 struct CORBA_any_type {
 	CORBA_TypeCode _type;
 	gpointer _value;
-	guint _flags;
+	CORBA_boolean _release;
 };
 
+typedef struct ORBit_marshal_value_info_struct {
+	CORBA_TypeCode alias_element_type;
+} ORBit_marshal_value_info;
+
 #define CORBA_ANYFLAGS_RELEASE 1
 
 
