@@ -177,8 +177,7 @@ CORBA_any *CORBA_exception_as_any(CORBA_Environment *ev)
 
 	ev->_any=g_new(CORBA_any, 1);
 	if(ev->_any!=NULL) {
-		memcpy(&(ev->_any->_type), &TC_CORBA_ExceptionDescription,
-			(size_t)sizeof(CORBA_TypeCode) );
+		ev->_any->_type = TC_CORBA_ExceptionDescription;
 		ev->_any->_value = ev->_params;
 		ev->_any->_release = 0;
 	}
@@ -214,11 +213,11 @@ void ORBit_handle_exception(GIOPRecvBuffer *rb, CORBA_Environment *ev,
 
 	rb->cur = ALIGN_ADDRESS(rb->cur, sizeof(len));
 	rb->decoder(&len, rb->cur, sizeof(len));
-	rb->cur += sizeof(len);
+	(guchar *)rb->cur += sizeof(len);
 
 	if(len) {
 		my_repoid = rb->cur;
-		rb->cur += len;
+		(guchar *)rb->cur += len;
 	} else
 		my_repoid = NULL;
 
@@ -226,7 +225,7 @@ void ORBit_handle_exception(GIOPRecvBuffer *rb, CORBA_Environment *ev,
 		ev->_major = CORBA_SYSTEM_EXCEPTION;
 		rb->cur = ALIGN_ADDRESS(rb->cur, sizeof(minor));
 		rb->decoder(&minor, rb->cur, sizeof(len));
-		rb->cur += sizeof(len);
+		(guchar *)rb->cur += sizeof(len);
 
 		for(minor = 1; exception_table[minor].repo_id; minor++) {
 			if(!strcmp(exception_table[minor].repo_id, my_repoid))
@@ -235,7 +234,7 @@ void ORBit_handle_exception(GIOPRecvBuffer *rb, CORBA_Environment *ev,
 
 		rb->cur = ALIGN_ADDRESS(rb->cur, sizeof(completion_status));
 		rb->decoder(&completion_status, rb->cur, sizeof(completion_status));
-		rb->cur += sizeof(completion_status);
+		(guchar *)rb->cur += sizeof(completion_status);
 
 		new=ORBit_alloc(sizeof(CORBA_SystemException), NULL, NULL);
 
