@@ -382,7 +382,7 @@ CORBA_ORB CORBA_ORB_init(int *argc, char **argv, CORBA_ORBid orb_identifier, COR
 
 	ORBit_chunks_init();
 
-	giop_init();
+	giop_init(argv[0]);
 
 	orb=g_new0(struct CORBA_ORB_type, 1);
 
