@@ -32,6 +32,7 @@
 #include <string.h>
 #include "orbit.h"
 #include "orbit_poa_type.h"
+#include "orbit_poa.h"
 
 static void ORBit_POAManager_release(PortableServer_POAManager poa_mgr,
 				     CORBA_Environment *ev);
@@ -292,7 +293,9 @@ ORBit_POA_new(CORBA_ORB orb,
 
 	poa->active_object_map = g_hash_table_new((GHashFunc)g_sequence_octet_hash,
 						  (GCompareFunc)g_sequence_octet_compare);
-
+	poa->objnum_to_obj = g_ptr_array_new();
+	g_ptr_array_set_size(poa->objnum_to_obj, 1);
+	g_ptr_array_index(poa->objnum_to_obj, 0) = NULL;
 
 	return poa;
 
@@ -376,7 +379,7 @@ ORBit_POA_handle_request(GIOPRecvBuffer *recv_buffer,
 	switch(poa->the_POAManager->state) {
 	case PortableServer_POAManager_HOLDING:
 		poa->held_requests = g_slist_prepend(poa->held_requests,
-						    recv_buffer);
+						     recv_buffer);
 		return;
 		break;
 	case PortableServer_POAManager_DISCARDING:
@@ -417,14 +420,23 @@ ORBit_POA_handle_request(GIOPRecvBuffer *recv_buffer,
 	servant = NULL;
 
 	if(poa->servant_retention == PortableServer_RETAIN) {
-		CORBA_sequence_octet oid;
-		oid._length = recv_buffer->message.u.request.object_key._length
-			- sizeof(CORBA_unsigned_long);
-		oid._buffer = recv_buffer->message.u.request.object_key._buffer
-			+ sizeof(CORBA_unsigned_long);
-		obj_impl = g_hash_table_lookup(poa->active_object_map,
-					       &oid);
-		if(obj_impl)
+		gpointer nptr;
+		CORBA_unsigned_long onum;
+
+		if(recv_buffer->message.u.request.object_key._length
+		   != sizeof(CORBA_unsigned_long) * 2) {
+			CORBA_exception_set_system(&ev,
+						   ex_CORBA_OBJECT_NOT_EXIST,
+						   CORBA_COMPLETED_NO);
+			goto errout;
+		}
+
+		nptr = recv_buffer->message.u.request.object_key._buffer;
+
+		onum = *((CORBA_unsigned_long *)((guchar *)nptr + sizeof(CORBA_unsigned_long)));
+		obj_impl = g_ptr_array_index(poa->objnum_to_obj, onum);
+		if(obj_impl
+		   && GPOINTER_TO_UINT(obj_impl) > poa->objnum_to_obj->len)
 			servant = obj_impl->servant;
 	}
 
@@ -478,24 +490,24 @@ ORBit_POA_find_POA_for_object_key(PortableServer_POA root_poa,
 	CORBA_unsigned_long pid;
 
 	pid = *((CORBA_unsigned_long *)key->_buffer);
-	g_assert(pid < root_poa->orb->poas->len);
-
-	return g_ptr_array_index(root_poa->orb->poas, pid);
+	if(pid < root_poa->orb->poas->len)
+		return g_ptr_array_index(root_poa->orb->poas, pid);
+	else
+		return NULL;
 }
 
 void
 ORBit_POA_find_object_key_for_oid(PortableServer_POA poa,
-				  PortableServer_ObjectId *oid,
+				  ORBit_POAObject *obj,
 				  CORBA_sequence_octet *retval)
 {
-	g_return_if_fail(poa && oid);
+	g_return_if_fail(poa && obj);
 
-	retval->_length = oid->_length + sizeof(CORBA_unsigned_long);
+	retval->_length = sizeof(CORBA_unsigned_long) * 2;
 	retval->_buffer = CORBA_octet_allocbuf(retval->_length);
 	CORBA_sequence_set_release(retval, CORBA_TRUE);
 	*((CORBA_unsigned_long *)retval->_buffer) = poa->poaID;
-	memcpy(retval->_buffer + sizeof(CORBA_unsigned_long),
-	       oid->_buffer, oid->_length);
+	*((CORBA_unsigned_long *)(retval->_buffer + sizeof(CORBA_unsigned_long))) = obj->objnum;
 }
 
 DEFINE_LOCK(id_assignment_counter);
@@ -615,7 +627,10 @@ void
 ORBit_POA_etherealize_objects(PortableServer_POA poa,
 			      CORBA_Environment *ev)
 {
-	EtherealizeInfo ei = {poa, ev};
+	EtherealizeInfo ei;
+
+	ei.poa = poa;
+	ei.ev = ev;
 
 	if(poa->servant_retention == PortableServer_RETAIN
 	   && poa->request_processing == PortableServer_USE_SERVANT_MANAGER) {
