@@ -65,7 +65,7 @@ typedef CORBA_char *CORBA_Identifier;
 
 typedef struct CORBA_ServiceDetail_type CORBA_ServiceDetail;
 typedef struct CORBA_Request_type *CORBA_Request;
-typedef struct CORBA_ServerRequest_type CORBA_ServerRequest;
+typedef struct CORBA_ServerRequest_type *CORBA_ServerRequest;
 typedef CORBA_char *CORBA_Context;
 typedef struct CORBA_DynFixed_type CORBA_DynFixed;
 typedef struct CORBA_DynAny_type CORBA_DynAny;
@@ -109,7 +109,7 @@ typedef struct CORBA_fixed_d_s {
 	CORBA_unsigned_short _digits;
 	CORBA_short _scale;
 	signed char _sign;
-	signed char _value[0];
+	signed char _value[1];
 } CORBA_fixed_d_s;
 
 #include "corba_env_type.h"
@@ -145,10 +145,6 @@ typedef struct CORBA_NVList_type {
 	GArray *list;
 } CORBA_NVList;
 
-struct CORBA_ServerRequest_type {
-	int fill_me_in;
-};
-
 typedef struct CORBA_DynAny_Invalid {
 	int dummy;
 } CORBA_DynAny_Invalid;
@@ -297,6 +293,7 @@ struct CORBA_DynArray_type {
 };
 
 struct CORBA_Current_type {
+	int fill_me_in;
 };
 
 #include "corba_portableserver_type.h"
