@@ -838,9 +838,18 @@ CORBA_Object PortableServer_POA_create_reference_with_id(PortableServer_POA obj,
 
 PortableServer_ObjectId *PortableServer_POA_servant_to_id(PortableServer_POA obj, PortableServer_Servant p_servant, CORBA_Environment *ev)
 {
+	PortableServer_ObjectId *retval, *orig;
 	PortableServer_ServantBase *serv = p_servant;
 	g_return_val_if_fail(p_servant != NULL, NULL);
-	return ORBIT_OBJECT_KEY(serv->_private)->object->object_id;
+
+	orig = ORBIT_OBJECT_KEY(serv->_private)->object->object_id;
+	retval = (PortableServer_ObjectId *)CORBA_sequence_octet__alloc();
+	retval->_length = retval->_maximum = orig->_length;
+	retval->_buffer = CORBA_octet_allocbuf(retval->_length);
+	memcpy(retval->_buffer, orig->_buffer, retval->_length);
+	CORBA_sequence_set_release(retval, CORBA_TRUE);
+
+	return retval;
 }
 
 CORBA_Object PortableServer_POA_servant_to_reference(PortableServer_POA obj, PortableServer_Servant p_servant, CORBA_Environment *ev)
