@@ -261,12 +261,14 @@ static RegisteredName *rn_unref(RegisteredName *nom)
     return NULL;
 }
 
-static void do_exit(void) { /* for alarm() handling */ _exit(1); }
+static void do_exit(void) {
+  syslog(LOG_DEBUG, "Got alarm\n");
+  _exit(5);
+}
 
 static gint
 server_is_alive(CORBA_Object server)
 {
-  GIOPConnection* cnx;
   int childpid, exitstatus, itmp;
 
   if(server->servant)
@@ -275,18 +277,27 @@ server_is_alive(CORBA_Object server)
   childpid = fork();
 
   if(!childpid) {
+    GIOPConnection* cnx = NULL;
+
+#if 0
+	volatile int stophere=1;
+	while(stophere);
+#elif 0
+	sleep(15);
+#endif
+
     signal(SIGALRM, (void (*)(int))do_exit);
-    alarm(1);
+    alarm(2);
     cnx = _ORBit_object_get_connection(server);
     syslog(LOG_DEBUG, "server_is_alive: cnx[%s] = %p\n",
 	   server->object_id, cnx);
-    _exit(cnx == 0);
+    _exit(cnx == NULL);
   }
 
   itmp = waitpid(childpid, &exitstatus, 0);
 
   if(itmp < 0) return FALSE;
-  return !exitstatus;
+  return !WEXITSTATUS(exitstatus);
 }
 
 static CosNaming_BindingIterator 
